# Conversion factors

POUNDS_OF_WATER_PER_GALLON = 8.34
BTU_PER_GALLON_PER_DEG_F = POUNDS_OF_WATER_PER_GALLON
BTU_PER_KWH = 3412
KWH_TH_PER_GALLON_PER_DEG_F = BTU_PER_GALLON_PER_DEG_F / BTU_PER_KWH
MINUTES_PER_HOUR = 60
BTU_PER_GALLON_OF_OIL = 140000
